<?php
// $Id: deadwood.conversions.inc,v 1.3 2008/07/26 13:39:10 solotandem Exp $

/**
 * @file
 * Generate version upgrade code from 5.x to 6.x.
 *
 * Copyright 2008 by Jim Berry ("solotandem", http://drupal.org/user/240748)
 */

/**
 * Convert files in a directory.
 *
 * @param string $dirname Module input directory containing the code files to be updated.
 * @param string $newdirname Module output directory where the converted code files will be written.
 * @param array $params Keyed array with run-time parameters. Should contain
 * these keys:
 *    'extensions' => array() // The file types to convert based on extension
 *    'conversions' => array() // The conversions to apply
 * @param boolean $recursive Indicates whether to recurse the subdirectories of $dirname.
 */
function deadwood_convert_dir($dirname, $newdirname, $params = array(), $recursive = TRUE) {
  static $ignore = array('.', '..', 'CVS', '.svn');

  // Create an output directory we can write to.
  if (!is_dir($newdirname)) {
    mkdir($newdirname);
  }
  else {
    deadwood_clean_directory($newdirname);
  }

  // Extract the file types to convert based on extension.
  $extensions = array();
  if (isset($params) && count($params) && array_key_exists('extensions', $params)) {
    $extensions = implode('|', $params['extensions']);
  }

  $dir = opendir($dirname);
  while ($file = readdir($dir)) {
    if (!in_array($file, $ignore)) {
      if (is_dir($dirname . '/' . $file)) {
        if ($recursive) {
          deadwood_convert_dir($dirname . '/' . $file, $newdirname . '/' . $file, $params, $recursive);
        }
      }
      // TODO Replace this preg_match with a php get file extension function.
      elseif (preg_match('@(?<!replace)(?<!settings)\.(' . $extensions . ')$@', $file)) {
        copy($dirname . '/' . $file, $newdirname . '/' . $file);
        drupal_set_message('Converting the file => ' . $file);
        deadwood_convert_file($newdirname . '/' . $file, $params);
      }
      else {
        copy($dirname . '/' . $file, $newdirname . '/' . $file);
      }
    }
  }
}

/**
 * Convert a file.
 *
 * @param string $filename The name of the file to convert.
 * @param array $params Keyed array with run-time parameters. Should contain
 * these keys:
 *    'extensions' => array() // The file types to convert based on extension
 *    'conversions' => array() // The conversions to apply
 */
function deadwood_convert_file($filename, $params = array()) {
  if (!file_exists($filename)) {
    return FALSE;
  }

  // Read the file and copy the contents.
  $cur = file_get_contents($filename);
  $new = $cur;

  // Extract the conversions to apply.
  $conversions = array();
  if (isset($params) && count($params) && array_key_exists('conversions', $params)) {
    $conversions = $params['conversions'];
  }

  // Process conversions on file.
  foreach ($conversions as $conversion) {
    if (function_exists($function = 'deadwood_convert_' . $conversion)) {
      $is_info_file = pathinfo($filename, PATHINFO_EXTENSION) == 'info';
      $is_info_conversion = strpos($conversion, 'info') === 0;
      // Only run info changes on info files
      // TODO Only run install changes on install files
      if ((!$is_info_file && !$is_info_conversion) || ($is_info_file && $is_info_conversion)) {
        call_user_func_array($function, array(&$new));
      }
    }
  }

  // Write the new file.
  if ($new != $cur) {
    if (!file_put_contents($filename, $new)) {
      drupal_set_message('File could not be written');
    }
    drupal_set_message('Replaced the file'); // Do we want this displayed?
  }
}

/**
 * Find the text of a particular function.
 *
 * TODO The match is dependent upon another function in the file after the
 * function being searched for.
 *
 * @param string $hook The suffix of the function name to find.
 *   Example: passing $hook = 'menu' will find the function
 *   whose name ends in '_menu'.
 * @param string $file The file to search.
 * @return string The function text.
 */
function deadwood_find_hook($hook, $file) {
  $pattern = '/^function \w+_' . $hook . '\(.*?(?=^function )/ms';
  preg_match($pattern, $file, $matches);

  return isset($matches[0]) ? $matches[0] : '';
}

/**
 * Loop on from and to arrays, converting the text of the subject string.
 *
 * @param string $from The pattern to search for.
 * @param string $to The string to replace the pattern with.
 * @param string $subject The string to search and replace.
 */
function deadwood_do_conversions($from, $to, &$subject) {
  for ($i = 0; $i < count($from); $i++) {
    $subject = preg_replace($from[$i], $to[$i], $subject);
  }
}

/**
 * Save the changes back to the file.
 *
 * @param string $cur The string to search for in $file.
 * @param string $new The replacement string.
 * @param string $file The text being replaced (all of or part of a file).
 * @param string $hook The hook being modified.
 */
function deadwood_save_changes($cur, $new, &$file, $hook) {
  if ($new != $cur) {
    $file = str_replace($cur, $new, $file);
    drupal_set_message('Completed conversions for ' . $hook);
  }
}

/**
 * Convert menu function.
 * See: http://drupal.org/node/103114.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_menu(&$file) {
  $hook = 'menu';
  $cur = deadwood_find_hook($hook, $file);
  $new = $cur;

  $msg1 = "/* TODO\n" .
          "   Non menu code that was placed in hook_menu under the '!\$may_cache' block\n" .
          "   so that it could be run during initialization, should now be moved to hook_init.\n" .
          "   Previously we called hook_init twice, once early in the bootstrap process, second\n" .
          "   just after the bootstrap has finished. The first instance is now called boot\n" .
          "   instead of init.\n" .
          "   \n" .
          "   In Drupal 6, there are now two hooks that can be used by modules to execute code\n" .
          "   at the beginning of a page request. hook_boot() replaces hook_init() in Drupal 5\n" .
          "   and runs on each page request, even for cached pages. hook_init() now only runs\n" .
          "   for non-cached pages and thus can be used for code that was previously placed in\n" .
          "   hook_menu() with \$may_cache = FALSE:\n" .
          "   \n" .
          "   Dynamic menu items under a '!\$may_cache' block can often be simplified\n" .
          "   to remove references to arg(n) and use of '%<function-name>' to check\n" .
          "   conditions. See http://drupal.org/node/103114.\n" .
          "   \n" .
          "   The title and description arguments should not have strings wrapped in t(),\n" .
          "   because translation of these happen in a later stage in the menu system.\n" .
          "*/\n";

  $from = array();
  $to = array();
  // hook_menu() no longer takes any parameters (remove $may_cache).
  $from[] = '/_menu\(\$may_cache/m';
  $to[] = '_menu(';
  $from[] = '/^(.*?if \((!|)\$may_cache\).*)$/m';
  $to[] = "$msg1$1";
  // callback becomes page callback.
  $from[] = '/\'callback\'/m';
  $to[] = '\'page callback\'';
  // callback arguments becomes page arguments.
  $from[] = '/\'callback arguments\'/m';
  $to[] = '\'page arguments\'';
  // access becomes access callback and access arguments.
  $from[] = '/\'access\' => user_access/m';
  $to[] = '\'access arguments\' => array(';
  $from[] = '/\'access\'/m';
  $to[] = '\'access arguments\'';
  $from[] = '/= user_access\(\'/m';
  $to[] = '= array(\'';
  // title and description arguments should not have strings wrapped in t().
//  $from[] = '/\'title\' => t\(\'(.*)\'\)([|,])$/m';
//  $to[] = '\'title\' => \'$1\'$2';
//  $from[] = '/\'description\' => t\(\'(.*)\'\)([|,])$/m';
//  $to[] = '\'description\' => \'$1\'$2';
  // There may be other t() references on these lines. Add a message?
  //  t\((('|)(\w+)('|))\)
//  $from[] = '/(\'title\' =>.*\W)t\(((\'|)([\w ]+)(\'|))\)/';
  $from[] = '/(\'title\' =>.*\W)t\((.*)\)/';
  $to[] = '$1$2';
  $from[] = '/(\'description\' =>.*\W)t\((.*)\)/';
  $to[] = '$1$2';
  // Repeat these as they may occur multiple times on a line (probably a way to do this with one call).
//  $from[] = '/(\'title\' =>.*\W)t\(((\'|)([\w ]+)(\'|))\)/';
  $from[] = '/(\'title\' =>.*\W)t\((.*)\)/';
  $to[] = '$1$2';
  $from[] = '/(\'description\' =>.*)\Wt\((.*)\)/';
  $to[] = '$1$2';
  // Menu index strings with wildcards may be simplified
  $from[] = '/(\'path\' =>.*)arg\(\d+\)(.*)$/m'; // '/(\'path\' =>.*)\..*arg\(\d+\).*\.(.*)$/m';
  $to[] = '$1\'%\'$2';

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert calls to url() and l().
 *
 * See: http://drupal.org/node/114774#url.
 * Adapted from the script at http://drupal.org/files/issues/replace.php_.txt.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_url(&$file) {
  $hook = 'url';
  $msg = "/* TODO\n" .
         "   Please manually fix the parameters on the l() or url() function on the next line.\n" .
         "   Typically, this was not changed because of a function call inside an array call like\n" .
         "   array('title' => t('View user profile.'))." .
         "*/\n";

  $callbacks = array('deadwood_fix_l', 'deadwood_fix_url');
  foreach ($callbacks as $callback) {
    $cur = $file;
    $new = '';

    // Process file in chunks to avoid PCRE bug?
    $pattern = $callback == 'deadwood_fix_l' ? '/(?=(?<!ion )(?<![a-zA-Z_\'\"])l\()/' : '/(?=(?<!ion )(?<![a-zA-Z_\'\"])url\()/';
    $chunks = preg_split($pattern, $cur);

    $search = $callback == 'deadwood_fix_l' ? '@(?<!ion )(?<![a-zA-Z_\'\"])l\((([^()]*|\((?2)?\))+)\)@' : '@(?<!ion )(?<![a-zA-Z_\'\"])url\((([^()]*|\((?2)?\))+)\)@';

    foreach ($chunks as $chunk) {
      $chg = preg_replace_callback($search, $callback, $chunk);
      $new .= $chg != '' ? $chg : $msg . $chunk;
    }

    deadwood_save_changes($cur, $new, $file, $hook);
  }
}

/**
 * Callback function for converting l() references.
 *
 * @param array $matches Function arguments.
 * @return Text of converted function call.
 */
function deadwood_fix_l($matches) {
  $args = deadwwod_get_args($matches[1]);

  // Two arguments, no change.
  if (count($args) <= 2) {
    return $matches[0];
  }

  $options = "array(";
  if (isset($args[2]) && $args[2][1] != 'array()' && $args[2][1] != 'NULL') {
    $options .= "'attributes' => " . $args[2][1] . ', ';
  }
  if (isset($args[3]) && $args[3][1] != "''" && $args[3][1] != '""' && $args[3][1] != 'NULL') {
    $options .= "'query' => " . $args[3][1] . ', ';
  }
  if (isset($args[4]) && $args[4][1] != "''" && $args[4][1] != '""' && $args[4][1] != 'NULL') {
    $options .= "'fragment' => " . $args[4][1] . ', ';
  }
  if (isset($args[5]) && $args[5][1] != "FALSE") {
    $options .= "'absolute' => TRUE, ";
  }
  if (isset($args[6]) && $args[6][1] != "FALSE") {
    $options .= "'html' => TRUE, ";
  }
  $options = rtrim($options, ', ');

  $return = 'l('. $args[0][1] . ', ' . $args[1][1] . ', ' . $options  . '))';
  return $return;
}

/**
 * Callback function for converting url() references.
 *
 * @param array $matches Function arguments.
 * @return Text of converted function call.
 */
function deadwood_fix_url($matches) {
  $args = deadwwod_get_args($matches[1]);

  // One argument, no change.
  if (count($args) <= 1) {
    return $matches[0];
  }

  // Ignore the url() call from l().
  if ($args[1][1] == '$options') return $matches[0];

  $options = "array(";
  if (isset($args[1]) && $args[1][1] != "''" && $args[1][1] != '""' && $args[1][1] != 'NULL') {
    $options .= "'query' => " . $args[1][1] . ', ';
  }
  if (isset($args[2]) && $args[2][1] != "''" && $args[2][1] != '""' && $args[2][1] != 'NULL') {
    $options .= "'fragment' => " . $args[2][1] . ', ';
  }
  if (isset($args[3]) && $args[3][1] != "FALSE") {
    $options .= "'absolute' => TRUE, ";
  }
  $options = rtrim($options, ', ');

  $return = 'url(' . $args[0][1] . ', ' . $options  . '))';
  return $return;
}

/**
 * Helper function for parsing arguments.
 *
 * @param array $matches Function arguments.
 * @return Text of converted function call.
 */
function deadwwod_get_args($string) {
  preg_match_all('` *(([^(),]+|\((([^()]*|\((?3)?\))+)\))+) *(,|$)`', $string, $matches, PREG_SET_ORDER);
  return $matches;
}

/**
 * Convert calls to taxonomy_node_get_terms(), taxonomy_node_get_terms_by_vocabulary(),
 * taxonomy_node_save and taxonomy_node_delete().
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_taxonomy(&$file) {
  $hook = 'taxonomy';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  /*
   * TODO Add this text if table name is found.
   * The {term_node} table now has a vid (version id) column, and (tid, nid, vid) is now the primary key.
   * Any direct queries against {term_node} should be checked, and probably modified to use the node
   * revision (vid) field instead of nid.
   *
  */
  // taxonomy_node_get_terms() now takes a full $node object, not just a nid (node id).
  $from[] = '/taxonomy_node_get_terms\s*\(\$(nid|node->nid)/';
  $to[] = 'taxonomy_node_get_terms($node';
//  $from[] = '/taxonomy_node_get_terms\(([^\$node]+)/m';
//  $to[] = '_menu(';
  // taxonomy_node_get_terms_by_vocabulary() now takes a full $node object, not just a nid (node id).
  $from[] = '/taxonomy_node_get_terms_by_vocabulary\s*\(\$(nid|node->nid)/m';
  $to[] = 'taxonomy_node_get_terms_by_vocabulary($node';
  // taxonomy_node_save() now takes a full $node object, not just a nid (node id).
  $from[] = '/taxonomy_node_save\s*\(\$(nid|node->nid)/m';
  $to[] = 'taxonomy_node_save($node';
  // taxonomy_node_delete() now takes a full $node object, not just a nid (node id).
  $from[] = '/taxonomy_node_delete\s*\(\$(nid|node->nid)/m';
  $to[] = 'taxonomy_node_delete($node';
  // taxonomy_node_delete_revision() has been added to delete all taxonomy terms from the current revision of the given $node object.

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert calls to format_plural() to allow for variable replacements like t().
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_format_plural(&$file) {
  $hook = 'format_plural';
  $cur = $file;
  $new = $cur;

  $msg = "/* TODO\n" .
         "   An argument for replacements has been added to format_plural(),\n" .
         "   escaping and/or theming the values just as done with t()." .
         "*/";

  $from = array();
  $to = array();
  // format_plural() accepts replacements.
  /* 5.x code:
   * strtr(format_plural($num, 'There is currently 1 %type post on your site.', 'There are currently @count %type posts on your site.'), array('%type' => theme('placeholder', $type)));
   * becomes 6.x code:
   * format_plural($num, 'There is currently 1 %type post on your site.', 'There are currently @count %type posts on your site.', array('%type' => $type));
   */
//  $from[] = '/[\s(]strtr\s*\(format_plural\s*\(.*\),\s*array\(.*=>\s*theme\(.*,(\w)*\)/';
//  $to[] = $msg . '$1'; // We could place the msg at the top of the file with other messages???
//  $from[] = '/format_plural\s*\(/';
//  $to[] = $msg . '$1';
  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert calls to hook_form_alter() for parameter changes.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_form_alter(&$file) {
  /*
   * TODO These module functions could have added additional parameters from the core signature.
   * We need to append any additional parameters to the end of the required parameters.
   * Should we search for the string '_form_alter' and place a notification message?
   * This might be present if the module called the hook itself.
   * Are there variables inside this function we could/should rename?
   */
  $hook = 'form_alter';
  $pattern = '/(?<!hook)_form_alter\s*(\(.*\))/';
  // No need for preg_match_all as a module should only define this function once?
  if (!preg_match($pattern, $file, $matches)) {
    return;
  }
  $cur = $matches[0];
  $new = $cur;

  $search = $matches[1];
  $replace = '(&$form, $form_state, $form_id)';
  $new = str_replace($search, $replace, $new);

  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert calls to hook_link_alter() for parameter changes.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_link_alter(&$file) {
  /*
   * TODO These module functions could have added additional parameters from the core signature.
   * We need to append any additional parameters to the end of the required parameters.
   * Should we search for the string '_link_alter' and place a notification message?
   * This might be present if the module called the hook itself.
   */
  $hook = 'link_alter';
  $pattern = '/_link_alter\s*(\(.*\))/';
  // No need for preg_match_all as a module should only define this function once?
  if (!preg_match($pattern, $file, $matches)) {
    return;
  }
  $cur = $matches[0];
  $new = $cur;

  $search = $matches[1];
  $replace = '(&$links, $node)';
  $new = str_replace($search, $replace, $new);

  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert calls to hook_profile_alter() for parameter changes.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_profile_alter(&$file) {
  /*
   * TODO These module functions could have added additional parameters from the core signature.
   * We need to append any additional parameters to the end of the required parameters.
   * Should we search for the string '_profile_alter' and place a notification message?
   * This might be present if the module called the hook itself.
   */
  $hook = 'profile_alter';
  $cur = deadwood_find_hook($hook, $file);
  $new = $cur;

  $from = array();
  $to = array();
  // Change function paramters.
  $from[] = '/_profile_alter\s*(\(.*\))/';
  $to[] = '_profile_alter(&$account)';
  // Change variable references.
  $from[] = '/\$fields\[/';
  $to[] = '$account->content[';

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert calls to hook_mail_alter() for parameter changes.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_mail_alter(&$file) {
  /*
   * TODO These module functions could have added additional parameters from the core signature.
   * We need to append any additional parameters to the end of the required parameters.
   * Should we search for the string '_mail_alter' and place a notification message?
   * This might be present if the module called the hook itself.
   */
  $hook = 'mail_alter';
  $cur = deadwood_find_hook($hook, $file);
  $new = $cur;

  $from = array();
  $to = array();
  // Change function paramters.
  $from[] = '/_mail_alter\s*(\(.*\))/';
  $to[] = '_mail_alter(&$message)';
  // Change variable references.
  $from[] = '/([^&])\$(mailkey|to|subject|body|from|headers)/';
  $to[] = '$1$message[\'$2\']';

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about need to register theme functions in hook_theme().
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_theme(&$file) {
  $hook = 'theme';
  $cur = $file;
  $new = $cur;

  $msg =
"/* TODO Implement the hook_theme registry. */
function example_theme() {
  return array(
    'something_format' => array(
      'file'      => 'something.inc',
      'template'  => 'something-format',
      'arguments' => array('content' => NULL),
    ),
  );
}";

  $from = array();
  $to = array();
  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about node/add automatic menu generation and alteration.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_node_add(&$file) {
  $hook = 'menu';
  $cur = deadwood_find_hook($hook, $file);
  $new = $cur;
  $hook = 'node_add';

  $msg =
"/* TODO
   The node/add/\$type menu items are now auto-generated by the menu system.
   You should not declare them in your menu hook. This means that you can
   use hook_menu_alter to change the visibility of an item or change the
   access callback. */";

  $from = array();
  $to = array();
  // node/add is now menu generated
  $from[] = "/^(.*\$items\[.*\'path\'\s*=>\s*\'node\/add.*?;)/ms";
  $to[] = "\n$msg\n\n$1";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about new hook_watchdog.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_hook_watchdog(&$file) {
  $hook = 'hook_watchdog';
  $cur = $file;
  $new = $cur;

  $msg =
"/* TODO
   There is a new hook_watchdog in core. This means that contributed modules
   can implement hook_watchdog to log Drupal events to custom destinations.
   Two core modules are included, dblog.module (formerly known as watchdog.module),
   and syslog.module. Other modules in contrib include an emaillog.module,
   included in the logging_alerts module. See syslog or emaillog for an
   example on how to implement hook_watchdog. */
function example_watchdog(\$log = array()) {
    if (\$log['severity'] == WATCHDOG_ALERT) {
      mysms_send(\$log['user']->uid,
        \$log['type'],
        \$log['message'],
        \$log['variables'],
        \$log['severity'],
        \$log['referer'],
        \$log['ip'],
        format_date(\$log['timestamp'])));
}";

  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert calls to watchdog() for parameter changes.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_watchdog(&$file) {
  $hook = 'watchdog';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  // Parameters of watchdog() changed
  // remove the t(.., array(.., function())) reference inside the watchdog() call
  $from[] = '/(watchdog\(.*),\s*t\((.*array.*?\)\))\)/';
  $to[] = "\$1, $2"; // TODO Are the '\' characters intended or leftover from above???
  // remove the t(.., array()) reference inside the watchdog() call
  $from[] = '/(watchdog\(.*),\s*t\((.*array.*?\))\)/';
  $to[] = "\$1, $2";
  // remove the t() reference inside the watchdog() call
  $from[] = '/(watchdog\(.*),\s*t\((.*?)\)/';
  $to[] = "\$1, $2";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert syntax for .info files.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_info1(&$file) {
  $hook = 'info1';
  $cur = $file;
  $new = $cur;

  // New syntax for .info files
  $search = '/(dependencies)\s*=\s*(.+)/';
  $callback = 'deadwood_fix_dependencies';
  $chg = preg_replace_callback($search, $callback, $new);
  $new = $chg != '' ? $chg : $new;

  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Specify core compatibility in .info files.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_info2(&$file) {
  $hook = 'info2';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  // Core compatibility now specified in .info files
  // TODO Revise expression to only make 1 change.
  $from[] = '/$/';
  $to[] = "\ncore = 6.x";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Specify PHP compatibility in .info files. (NOT DONE)
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_info3(&$file) {
//  $hook = 'info3';
//  $cur = $file;
//  $new = $cur;

//  $from = array();
//  $to = array();
//  // Core compatibility now specified in .info files
//  $from[] = '/(dependencies\s*=.*)$/m';
//  $to[] = "$1\ncore = 6.x";
//
//  deadwood_do_conversions($from, $to, $new);
//
//  // New syntax for .info files
//  $search = '/(dependencies)\s*=\s*(.+)/';
//  $callback = 'deadwood_fix_dependencies';
//  $chg = preg_replace_callback($search, $callback, $new);
//  $new = $chg != '' ? $chg : $new;
//
//  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Callback function for converting dependencies.
 *
 * @param array $matches Function arguments.
 * @return Text of converted function call.
 */
function deadwood_fix_dependencies($matches) {
  // Two arguments, no change.
  if (count($matches) <= 2) {
    return $matches[0];
  }

  $deps = explode(' ', $matches[2]);
  $new = array();
  foreach ($deps as $dep) {
    $new[] = $matches[1] . '[] = ' . $dep;
  }
  $new = implode("\n", $new);
  return $new;
}

/**
 * Convert calls to cache_set() for parameter changes.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_cache1(&$file) {
  $hook = 'cache1';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  // Changes to cache_set parameter order
  // remove a serialize() reference inside the cache_set() call
  // switch order of parameters 2 and 3
  $from[] = '/cache_set\s*\(\s*(.+),\s*(.+),\s*(serialize\s*\(|)(.+)(\)|)\)/';
  $to[] = "cache_set($1, $4, $2)";
  // remove 'cache' as the table parameter since it is defaulted
  $from[] = '/cache_set\s*\(\s*(.+),\s*(.+),\s*(\'cache\')\)/';
  $to[] = "cache_set($1, $2)";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Remove (un)serialize calls with cache_get() and cache_set().
 * This is not a perfect split on the conversion since we remove
 * the serialize reference in cache1 above.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_cache2(&$file) {
  $hook = 'cache2';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  // Cache set and get automatically (un)serialize complex data types
  $from[] = '/unserialize\s*\(\s*cache_get/';
  $to[] = "cache_get";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about new user_mail_tokens.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_mail_tokens(&$file) {
  $hook = 'mail_tokens';
  $cur = $file;
  $new = $cur;

  $msg =
"/* TODO New user_mail_tokens() method may be useful.
   user.module now provides a user_mail_tokens() function to return an array
   of the tokens available for the email notification messages it sends when
   accounts are created, activated, blocked, etc. Contributed modules that
   wish to make use of the same tokens for their own needs are encouraged
   to use this function. */";

  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace calls to $_SERVER['REMOTE_ADDR'] with new function.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_ip_address(&$file) {
  $hook = 'ip_address';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  // New ip_address() function when working behind proxies
  $from[] = '/\$_SERVER\[\'REMOTE_ADDR\'\]/';
  $to[] = "ip_address()";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to file_check_upload().
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_upload1(&$file) {
  $hook = 'upload1';
  $cur = $file;
  $new = $cur;

  $msg =
"/* TODO Modify the validators array to suit your needs.
   This array is used in the revised file_save_upload */
  \$validators = array(
    'file_validate_is_image' => array(),
    'file_validate_image_resolution' => array('85x85')),
    'file_validate_size' => array(30 * 1024),
  );";

  $from = array();
  $to = array();
  // file_check_upload() merged into file_save_upload()
  // file_save_upload() has new parameter
  $from[] = '/^(.*)(file_check_upload)\s*\((.*)\)/m';
  $to[] = "$msg\n\n$1file_save_upload($2, \$validators)";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to {file_revisions} table.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_upload2(&$file) {
  $hook = 'upload2';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  // The {file_revisions} table is now {upload}
  $from[] = '/{file_revisions}/';
  $to[] = "{upload}";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about displaying node form elements in previews.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_node_preview(&$file) {
  $hook = 'node_preview';
  $cur = $file;
  $new = $cur;

  $msg =
"/* TODO Node previews and adding form fields to the node form.
   There is a subtle but important difference in the way node previews (and other
   such operations) are carried out when adding or editing a node. With the new
   Forms API, the node form is handled as a multi-step form. When the node form
   is previewed, all the form values are submitted, and the form is rebuilt with
   those form values put into \$form['#node']. Thus, form elements that are added
   to the node form will lose any user input unless they set their '#default_value'
   elements using this embedded node object. */";

  $from = array();
  $to = array();
  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to custom_url_rewrite with new functions.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_custom_url(&$file) {
  $hook = 'custom_url';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();
  // custom_url_rewrite('alias', ..) is now custom_url_rewrite_outbound
  // Change function_exists calls.
  $from[] = "/^(\s*if.*exists)(\('custom_url_rewrite'\))(.*\('alias.*?;)/ms";
  $to[] = "$1('custom_url_rewrite_outbound')$3";
  // Change function calls.
  $from[] = "/(custom_url_rewrite)(\('alias', )/";
  $to[] = "$1_outbound(";
  // custom_url_rewrite('source', ..) is now custom_url_rewrite_inbound
  // Change function_exists calls.
  $from[] = "/^(\s*if.*exists)(\('custom_url_rewrite'\))(.*\('source.*?;)/ms";
  $to[] = "$1('custom_url_rewrite_inbound')$3";
  // Change function calls.
  $from[] = "/(custom_url_rewrite)(\('source', )/";
  $to[] = "$1_inbound(";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace paramters for hook_help.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_hook_help(&$file) {
  $hook = 'help';
  $cur = deadwood_find_hook($hook, $file);
  $new = $cur;
  $hook = 'hook_help';

  $from = array();
  $to = array();

  // Save the parameter name to use in next search.
  $old_param = '$section';
  $pattern = "/_help\((.+)\)/";
  preg_match($pattern, $cur, $matches);
  if (isset($matches[1])) {
    $old_param = $matches[1];
  }
  // hook_help() parameters are changed
  // Change function parameter signature.
  $from[] = "/_help\((.+)\)/";
  $to[] = "_help(\$path, \$arg)";
  // Change function body.
  $from[] = "/\\$old_param/";
  $to[] = "\$path";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to taxonomy_get_vocabulary with taxonomy_vocabulary_load.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_taxonomy_vocabulary(&$file) {
  $hook = 'taxonomy_vocabulary';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();

  // taxonomy_get_vocabulary() changed to taxonomy_vocabulary_load().
  $from[] = '/taxonomy_get_vocabulary\s*\(/';
  $to[] = 'taxonomy_vocabulary_load(';

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to hook_init with hook_boot.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_hook_init(&$file) {
  $hook = 'hook_init';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();

  // hook_init() changed to hook_boot().
  $from[] = '/hook_init\s*\(/';
  $to[] = 'hook_boot(';
  // Added a message to deadwood_convert_menu about new hook_init.

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to access_control with permissions.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_access_control(&$file) {
  $hook = 'access_control';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();

  // Access control renamed to permissions.
  $from[] = '/(user_access\s*\(.*)administer access control/';
  $to[] = "$1administer permissions";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to locale_refresh_cache with cache_clear_all.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_locale_cache(&$file) {
  $hook = 'locale_cache';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();

  // Access control renamed to permissions.
  $from[] = '/locale_refresh_cache\s*\(\)/';
  $to[] = "cache_clear_all('locale:', 'cache', TRUE)";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about FormAPI image buttons.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_image_buttons(&$file) {
  $hook = 'image_buttons';
  $cur = $file;
  $new = $cur;

  $msg =
"/* TODO FormAPI image buttons are now supported.
   FormAPI now offers the 'image_button' element type, allowing developers to
   use icons or other custom images in place of traditional HTML submit buttons.

\$form['my_image_button'] = array(
  '#type'         => 'image_button',
  '#title'        => t('My button'),
  '#return_value' => 'my_data',
  '#src'          => 'my/image/path.jpg',
); */";

  $from = array();
  $to = array();
  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about db_last_insert_id.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_db_last(&$file) {
  $hook = 'db_last';
  $cur = $file;
  $new = $cur;

  $msg =
"/* TODO db_next_id() is gone, and replaced as db_last_insert_id()
   Since db_next_id() introduce some problems, and the use of this function
   can be replaced by database level auto increment handling, db_next_id()
   is now gone and replaced as db_last_insert_id() with help of serial type
   under Schema API (check out http://drupal.org/node/149176 for more details).
   Please refer to drupal_write_record() as demonstration. */";

  $from = array();
  $to = array();
  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Replace references to admin/logs with admin/reports.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_admin_logs(&$file) {
  $hook = 'admin_logs';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();

  // admin/logs renamed to admin/reports.
  $from[] = '/admin\/logs/';
  $to[] = "admin/reports";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add note about drupal_set_breadcrumb.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_breadcrumb(&$file) {
  $hook = 'breadcrumb';
  $cur = $file;
  $new = $cur;

  // Check for calls to menu_set_location.
  $pattern = "/menu_set_location/";
  if (!preg_match($pattern, $cur, $matches)) {
    return;
  }

  $msg =
"/* TODO Use drupal_set_breadcrumb() instead of menu_set_location() to set
   custom breadcrumbs.
   Currently in D5, menu_set_location() is 'misused' in several modules to set
   a custom breadcrumb, drupal_set_breadcrumb() should be used instead, as
   discussed in #177497.
   Note, when using drupal_set_breadcrumb(), you need to include 'home' but
   not the current page.
   Alternatively, if you do want to set the current location in the menu tree
   as well as affect breadcrumbs, use menu_set_item(). */";

  $from = array();
  $to = array();
  // Find the Id line in the file.
  $from[] = '/^(\/\/ \$Id.*\$$)/m';
  $to[] = "$1\n\n$msg";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Convert parameters in references to user_authenticate.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_user_authenticate(&$file) {
  $hook = 'user_authenticate';
  $cur = $file;
  $new = $cur;

  $from = array();
  $to = array();

  // user_authenticate changed parameters.
  $from[] = '/(user_authenticate)\s*\(\s*(.*),\s*(.*?)\)/';
  $to[] = "$1(array('name' => $2, 'pass' => $3)";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Add a parameter in references to hook_access.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_hook_access(&$file) {
  $hook = 'access';
  $cur = deadwood_find_hook($hook, $file);
  $new = $cur;
  $hook = 'hook_access';

  $from = array();
  $to = array();

  // hook_access() added a parameter.
  $from[] = '/(_access)\s*\(\s*(.*?)\)/';
  $to[] = "$1($2, \$account)";
  // remove global user reference.
  $from[] = '/^.*(global\s*\$user;)$/m';
  $to[] = "";
  // rename user to account.
  $from[] = '/\$user/';
  $to[] = "\$account";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}

/**
 * Remove javacript inclusions in hook_footer.
 *
 * @param string $file The file to convert.
 */
function deadwood_convert_hook_footer(&$file) {
  $hook = 'footer';
  $cur = deadwood_find_hook($hook, $file);
  $new = $cur;
  $hook = 'hook_footer';

  $msg = "/* TODO It's no longer possible to add javascript to header in hook_footer(). */";

  $from = array();
  $to = array();

  // Can't add javascript to header in hook_footer.
  $from[] = '/^(.*drupal_add_js\s*\(.*)$/m';
  $to[] = "$msg\n//  $1";

  deadwood_do_conversions($from, $to, $new);
  deadwood_save_changes($cur, $new, $file, $hook);
}
